#!/bin/csh -f
######################################################
#
# Convert pdf/postscript to odb++
#
# This script wraps the call to pstoedit
#
# Ben Michelson Aug 2009
# Updated Mar 2011
# Copyright 2009,2011 Frontline PCB Solutions LP
#
######################################################

######################################################
#
# Parse input parameters
#
######################################################
@ i = 1
while ($i < $#argv) 
   set key = $argv[$i]
   @ i++
   set val = "$argv[$i]"
   @ i++
   if ($?version) then
      set key = $key:s/-//
   else
      set key = `echo $key | sed s/^-//g`
   endif
   eval set P_$key = \"$val\"
end

if ($?P_inp) then
   set in_file = "$P_inp"
else
   echo Input file not specified. Expected -inp
   exit 1
endif

if ($?P_out) then
   set out = "$P_out"
else
   echo Output directory not specified. Expected -out
   exit 1
endif

if ($?P_ttf) then
   set ttf_font_path = "$P_ttf"
endif

if ($?P_bg) then
   set cutoff = $P_bg
else
   set cutoff = 200
endif

if ($?P_log) then
   set log = $P_log
else
   set log = $P_out:h"/log.$$"
endif

######################################################
#
# Find location of ghostscript
#
######################################################
if (! $?GS) then
   if (`uname` == HP-UX) then
      setenv GS gs
   else
      unalias gs
      set gs = (`which gs`)
      if ($#gs == 1) then
         setenv GS $gs
      else
         set gs = (`which gswin32c`)
         if ($#gs == 1) then
            setenv GS $gs
         endif
      endif
   endif
endif


######################################################
#
# Define default True Type font to use
#
######################################################
# setenv GENESIS_TTF_DEFAULT Vera


######################################################
#
# Define path of conversion table from Font name to
# Font file
#
######################################################
# setenv TTF2FONT $GENESIS_EDIR/r2v/ttf2font.txt

######################################################
#
# Define cutoff point between positive and negative
# (0,1) where 0 = black and 1 = white
#
# Convert from [0-255] to [1-0]
# 
######################################################
alias MATH 'set \!:1 = `echo "\!:3-$" | bc -l`'
MATH cutoff = (255 - $cutoff) / 255
setenv POS_NEG_CUTOFF $cutoff


######################################################
#
# Define semicolon separated list of paths containing fonts
# ($WINDIR/Fonts is automatically used as default on Windows)
#
######################################################
if ($?ttf_font_path) then
   setenv GENESIS_TTF_PATH "$ttf_font_path"
endif


######################################################
#
# Check $out
#
######################################################
if (! -e "$out:h") then
   mkdir -p "$out:h"
   if (! -d "$out:h") then
      echo $out:h does not exist and cannot be created
      exit 1
   endif
endif

touch "$out"
if (-e "$out") then
   rm "$out"
else
   echo No write permission for $out
   exit 1
endif

######################################################
#
# Define POTRACE_PARAMS
#
# For more information on POTRACE parameters
# see http://potrace.sourceforge.net/#usage
#
# Other parameters might be
# --alphamax 0.2  --unit 10
######################################################
if (! $?POTRACE_PARAMS) then
   setenv POTRACE_PARAMS "--blacklevel $POS_NEG_CUTOFF --turdsize 0"
endif
setenv POTRACE_PARAMS "--blacklevel .5 --turdsize 0"

######################################################
#
# Allow pstoedit to find shared libraries
#
######################################################
setenv SHLIB_PATH      $cwd
setenv LD_LIBRARY_PATH $cwd

######################################################
#
# Call translation program
#
######################################################
# set verbose
(./pstoedit -ndt "$in_file" $out.%d -f odb) >& $log

######################################################
#
# Try to fix input file and retranslate
#
######################################################
if ($status && $?GS) then

   if (! $?GENESIS_TMP) then
      setenv GENESIS_TMP $GENESIS_DIR/tmp
   endif

   ###################################################
   #
   # gswin32c.exe does not support UNC paths
   #
   ###################################################
   if ("$in_file" =~ //*) then
      set tmp_in_file = $GENESIS_TMP/ps_tmp$$.ps
      cp  "$in_file" "$tmp_in_file"
      set in_file = "$tmp_in_file"
   endif

   set new_in_file = $GENESIS_TMP/ps$$.ps
   "$GS" -q -dNOPAUSE -dBATCH -dSAFER -sDEVICE=pswrite "-sOutputFile=$new_in_file" -c save pop -f "$in_file"

   if ($?tmp_in_file) then
      rm "$tmp_in_file"
   endif

   if (-e $new_in_file) then
      (./pstoedit -ndt "$new_in_file" $out.%d -f odb) >& $log
      rm $new_in_file
   endif

endif

chdir "$out:h"

######################################################
#
# Remove png files generated
#
######################################################
set nonomatch
rm -f *.png

######################################################
#
# Remove last file (erroneously generated by pstoedit)
#
######################################################
if (-e features.1) then
   foreach file (features.*)
      if (-z $file) then
         rm $file
      else
         set first_line = `head -1 $file`
         if ("$first_line" == "# End of file") then
            rm $file
         endif
      endif
   end
endif

######################################################
#
# rename feature files from pstoedit naming conversion
# to Genesis 2000 naming convention
#
######################################################
if (-e features.1) then
   foreach file (features.*)
      set i = $file:e
      @ i--;
      mv $file $file:r"_"$i
   end
else
   exit 1
endif
