#---------------------------------------------------
#
#  Module : tclu.tcl
#  Author : MK
#  Date   : 20 Aug 1997
#
#  Description :
#
#  This module provides general utilities needed by the Frontline Pro-Installer
#  module and the various installation plug-ins.
#  All the public procedures are called tclu_<name>
#  All the private procedures are called tclu.<name> - should never be called
#
#  List of public procedures in this module:
#
#  tclu_set_tools_dir - 
#  tclu_set_os        -   Initialization routines
#  tclu_set_commands  -
#  tclu_get_genesis_epath - Specific for Enterprise/Genesis
#  tclu_get_genesis_path  - Specific for Enterprise/Genesis
#  tclu_get_home    - 	Get the HOME environment variable 
#  tclu_get_user    -   Get the current user
#  tclu_break_line  -	Breaks a line at convenient places
#  tclu_is_path_dir -   Checks if a path is or links to a directory
#  tclu_is_path_file -	Checks if a path is or links to a file
#  tclu_is_path_executable - Checks if a path is executable
#  tclu_is_path_dot -   Checks if a path is . or ..
#  tclu_mkdir -		Creates a directory with certain precautions
#  tclu_get_dir_for_file - Returns the directory of a file
#  tclu_get_bak_name -  Create a date/time tagged backup name
#  tclu_get_tmp_dir  -  Get the directory for temp files
#  tclu_get_tmp_name -  Create a temporary file names
#  tclu_copy_txt_file - Copies a text file (replacing lf to crlf if needed)
#  tclu_del_tmp_files - Deletes all the process temp files
#  tclu_check_files   - Checks a list of files against a table
#  tclu_is_root       - Checks if current user is root
#  tclu_file_exists   - Check if a file exists (Needed for SunOS)
#  tclu_path_back_to_fore - Converts back slashes to forward slashes
#  tclu_str_cmp_nc    - Compare 2 strings (case insensitive)
#  tclu_get_hostid -	Gets the host id of the workstation
#  tclu_get_dongle -    Gets the id of the dongle
#  tclu_get_gz_size -   calculates the size of a gzipped file
#  tclu_log_set_path -  Sets the path of the log file
#  tclu_log_get_path -  Gets the path of the log file
#  tclu_log_newline -   
#  tclu_log_message -	Procedures supporting writing of a log file
#  tclu_log_file -
#  tclu_create_lst_file - Creates a .lst file for a directory
#  tclu_check_lst_file  - Verifies a .lst file against a directory
#  tclu_check_sum_file - Checks a .sum file against the file
#  tclu_get_checksum -   Creates a md5sum
#  tclu_get_dat_lst_file 
#  tclu_update_file -     |
#  tclu_find_str_in_file -|--- Find/Update text files
#  tclu_put_after         |
#  tclu_put_at_eof        |
#  tclu_set_system_env  - Sets a systemwide environment variable (NT only)
#  tclu_null_file       - path of null file (/dev/null or nul)
#---------------------------------------------------
#
#  Change History 
#  --------------
#  970820 - First Version
#  970906 - Documentation
#

global TCLU_G
set    TCLU_G(IS_ROOT) 0

#*******************************************************
#
# Name :	tclu_set_tools_dir
#
# Description :	
#   Sets the locations of the tools directory 
#
# Input	:	TOOLS_DIR - location of the tools
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tclu_set_tools_dir {TOOLS_DIR} {
  global TCLU_G
  
  set TCLU_G(TOOLS) $TOOLS_DIR
}

#*******************************************************
#
# Name :	tclu_set_os
#
# Description :	
#   Sets the operating system
#
# Input	:	OS - name of operating system
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tclu_set_os {OS} {
  global TCLU_G
  
  set TCLU_G(OS) $OS
}

#*******************************************************
#
# Name :	tclu_set_commands
#
# Description :	
#   Sets the locations of the some useful commands
#
# Input	:	CAT   - Path to cat command
#               CHMOD - Path to chmod command
#               CP    - Path to cp command
#               DD    - Path to dd command
#               TAR   - Path to tar command
#               WHOAMI - Path to whoami command
# Output :	None
# Return Value:	HOME environment variable
#
#*******************************************************
#
proc tclu_set_commands {CAT CHMOD CP DD TAR WHOAMI} {
  global TCLU_G
  
  set TCLU_G(CAT)    $CAT
  set TCLU_G(CHMOD)  $CHMOD
  set TCLU_G(CP)     $CP
  set TCLU_G(DD)     $DD
  set TCLU_G(TAR)    $TAR
  set TCLU_G(WHOAMI) $WHOAMI
}


#*******************************************************
#
# Name :	tclu_get_home
#
# Description :	
#   Returns the value of the HOME environment variable
#
# Input	:	None
# Output :	None
# Return Value:	HOME environment variable
#
#*******************************************************
#
proc tclu_get_home {} {
   global env
   global TCLU_G

   if [info exists env(HOME)] {
      return $env(HOME)
   } else {
      if [info exists env(USERPROFILE)] {
         return "$env(USERPROFILE)"
      } else {
         if [info exists env(windir)] {
            return "[string index $env(windir) 0]:/"
         } else {
            return "/"
         }
      }
   }
}

#*******************************************************
#
# Name :	tclu_get_user
#
# Description :	
#   Returns the current user of the system
#
# Input	:	None
# Output :	None
# Return Value:	User name
#
#*******************************************************
#
proc tclu_get_user {}  {
   global TCLU_G
   global env
    
   if {$TCLU_G(OS) == "nt"} {
      return $env(USERNAME)
   } else {
      return [exec $TCLU_G(WHOAMI)]
   }
}

#*******************************************************
#
# Name :	tclu_get_genesis_epath
#
# Description :	
#   Returns the value of the genesis executable directory
#
# Input	:	None
# Output :	None
# Return Value:	HOME environment variable
#
#*******************************************************
#
proc tclu_get_genesis_epath {} {
   global env
   
   set GENESIS_DIR [tclu_get_genesis_path]
   if [info exists env(GENESIS_EDIR)] {
      set EDIR $env(GENESIS_EDIR)
      if [tclu_is_abs_path $EDIR] {
         return $EDIR
      }
   } else {
     if [info exists env(GENESIS_VER)] {
         set EDIR e$env(GENESIS_VER)
     } else {
	 # Should not happen
	 set EDIR e   
     }
   }
   return [file join $GENESIS_DIR $EDIR]   
}

#*******************************************************
#
# Name :	tclu_get_genesis_path
#
# Description :	
#   Returns the value of the genesis home directory
#
# Input	:	None
# Output :	None
# Return Value:	GENESIS_DIR environment variable or /genesis
#
#*******************************************************
#
proc tclu_get_genesis_path {} {
   global env
   global TCLU_G
   
   if [info exists env(GENESIS_DIR)] {
      return $env(GENESIS_DIR)
   } else {
      if {$TCLU_G(OS) == "nt"} {
         return "$env(HOMEDRIVE)/genesis"
      } else {
         return "/genesis"
      }
   }
}

#*******************************************************
#
# Name :	tclu_is_abs_path
#
# Description :	
#   Returns 1 if the path is absolute (Starts with / on UNIX or
#   with a X: on NT.
#
# Input	:	None
# Output :	None
# Return Value:	1 for absolute, 0 for relative
#
#*******************************************************
#
proc tclu_is_abs_path { PATH } {
   global TCLU_G
   
   set ABS 0
   if {$TCLU_G(OS) == "nt"} {
      if { [string length $PATH] >=3 } {
         if { [string index $PATH 1] == ":" } {
	    set ABS 1
	 }
      }
   } else {
      if { [string length $PATH] >=1 } {
         if { [string index $PATH 0] == "/" } {
	    set ABS 1
	 }
      }      
   }
   return $ABS
}

################ STRING UTITLITIES #######################
#*******************************************************
#
# Name :	tclu_break_line
#
# Description :	
#   Breaks a long string to lines according to MAXLINE.
#   Break occurs only on blank character.
#   If there is no blank to meet the criteria, string will 
#   be broken after MAXLINE characters.
#
# Input	:	S - The string to break
#               MAXLINE - The maximal characters in a line
# Output :	None
# Return Value:	New string
#
#*******************************************************
#
proc tclu_break_line {S MAXLINE} {
  set S1 $S
  set SNEW ""
  while {1} {
     set LEN [string length $S1]
     if { $LEN > $MAXLINE } {
        set BREAK_INDEX [expr $MAXLINE-1]
	while {$BREAK_INDEX > 0} {
	   set CHAR [string range $S1 $BREAK_INDEX $BREAK_INDEX]
	   if {[string compare $CHAR " "] == 0} {
	     incr BREAK_INDEX 
	     break 
	   }
	   incr BREAK_INDEX -1
	}
	if {$BREAK_INDEX == 0} {
	   set BREAK_INDEX $MAXLINE
	}
	append SNEW [string range $S1 0 [expr $BREAK_INDEX-1]] "\n"
	set S1 [string range $S1 $BREAK_INDEX end]
     } else {
	append SNEW $S1
	break 
     }
  }
  return "$SNEW"
}

################ FILE UTITLITIES #######################

#*******************************************************
#
# Name :	tclu_is_path_dir
#
# Description :	
#   Returns 1 if the file is a directory. It is better than
#   'file isdirectory' built-in since it also handles the case where the 
#   path is a link to a directory
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is a directory
#		0 - otherwise
#*******************************************************
#
proc tclu_is_path_dir { PATH } {
  set TYPE ""

  if [tclu_file_exists $PATH] {
    set TYPE [file type $PATH]
  }
  if { $TYPE == "link" } {
    set TYPE [file type [tclu.transfer_link $PATH]]
  }
  if { $TYPE == "directory" } {
    return 1
  } else {
    return 0
  }
}

#*******************************************************
#
# Name :	tclu_is_path_file
#
# Description :	
#   Returns 1 if the file is a file. It is better than
#   'file isfile' built-in since it also handles the case where the 
#   path is a link to a file
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is a file
#		0 - otherwise
#*******************************************************
#
proc tclu_is_path_file { PATH } {
   set TYPE ""
  if { [string compare $PATH ""] == 0 } {
    return 0
  }
  
  if [tclu_file_exists $PATH] {
      set TYPE [file type $PATH]
   }
   if { $TYPE == "link" } {
      set TYPE [file type [tclu.transfer_link $PATH]]
   }
   if { $TYPE == "file" } {
      return 1
   } else {
      return 0
   }
}

#...............................................................
proc tclu.transfer_link { PATH } {
   global TCLU_G
      
   set TYPE [file type $PATH]   
   if { $TYPE != "link" } {
      return $PATH
   }
   
   set LINK [file readlink $PATH]
   
   if { $TCLU_G(OS) == "nt" } {
      return $LINK 
   }
   if { [string index $LINK 0] == "/" } {
     # Absolute path #
      return $LINK       
   } else {
     # Relative path #
     set DIR [file dirname $PATH]
     set FINAL [file join $DIR $LINK]
   }
   return $FINAL
}

#*******************************************************
#
# Name :	tclu_is_path_executable
#
# Description :	
#   Returns 1 if the file is a file. It is better than
#   'file executable' built-in since it also handles the case where the 
#   path omits final ".exe" for Windows NT
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is executable
#		0 - otherwise
#*******************************************************
#
proc tclu_is_path_executable { PATH } {
  global TCLU_G
  
  if { $TCLU_G(OS) == "nt" } {
     if { [regexp -nocase "\\.exe$" $PATH] } {
        return [file executable $PATH]
     } else {
        return [file executable "$PATH.exe"]
     }
   } else {
      return [file executable $PATH]
   }
}

#*******************************************************
#
# Name :	tclu_is_path_dot
#
# Description :	
#   Returns 1 if the tail of the name is . or ..
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is a dot(dot) file
#		0 - otherwise
#*******************************************************
#
proc tclu_is_path_dot { PATH } {
   set TAIL [file tail $PATH]
   if {$TAIL == "." || $TAIL == ".."} {
      return 1
   }
   return 0
}

#*******************************************************
#
# Name :	tclu_is_path_hidden
#
# Description :	
#   Returns 1 if the tail of the name starts with a dot
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is hidden
#		0 - otherwise
#*******************************************************
#
proc tclu_is_path_hidden { PATH } {
   set TAIL [file tail $PATH]
   if [string match .* $TAIL ] {
      return 1
   }
   return 0
}

#*******************************************************
#
# Name :	tclu_mkdir
#
# Description :	
#   Creates a directory under certain checks and precautions:
#   - Path must contain only legal characters:
#       letters, digits, dot, underscore, colon, slash, dash, backslash
#   - If the path exists and is a directory, a check is made that it is
#     writable (Error is popped up if not)
#   - An error is popped up and returned if the path is to an existing file
#   - If the directory does not exists, a confirmation message is popped up
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	1 - if path is a file
#		0 - otherwise
#*******************************************************
#
proc tclu_mkdir { PATH {QUIET 0} } {
  global TCLU_G
  if {$TCLU_G(OS) == "nt"} {
     set LEGAL {^[a-zA-Z]:[\\a-z A-Z0-9._:/\-]+$}
#    Ignores regular paths & UNC paths
  } else {
     set LEGAL {^[\\a-zA-Z0-9._:/\-]+$}
  }
  set PATH [string trim $PATH]
#
# Check path characters legality
#
  if {! [regexp $LEGAL $PATH]} {
     tku_popup_message error ok \
	"Path contains illegal characters\
	 \n\nName  = $PATH\n"
     return 1
  }
  set EXISTS [tclu_file_exists $PATH]
  set IS_DIR [tclu_is_path_dir $PATH]
#
#  If path is an exiting directory, check its permissions
#     
  if { $EXISTS && $IS_DIR } {
     set WRITABLE [file writable $PATH]
     if { ! $WRITABLE } {
       tku_popup_message error ok \
  	  "Directory is not writable\
	   \n\nName  = $PATH\n"
        return 1
     }
     return 0
  }
#
#  If path is an existing file, it is an error
#  
  if { $EXISTS && ! $IS_DIR } {
     tku_popup_message error ok \
	"Path is an existing file\
	 \n\nName  = $PATH\n"
     return 1
  }
#
#  Confirm creation of a directory and handle any creation errors  
#  
  if { ! $EXISTS } {
    if {$QUIET == 0} {
      set CHOICE \
	  [tku_popup_message question yesno \
	       "No such directory\
       \n\nName=$PATH\n\nCreate ?\n"]
    }
    if {$QUIET != 0 || $CHOICE == "yes"} {
       set STATUS [catch {file mkdir $PATH} ERROR]
       if { $STATUS } {
         tku_popup_message error ok \
	   "Directory creation failure\
	    \n\nName  = $PATH\
	    \n\nError = $ERROR\n"
	 return 1
       }
    } else {
       return 1 
    }
  }  
  return 0 
}

#*******************************************************
#
# Name :	tclu_get_dir_for_file
#
# Description :	
#   For a dicrectory - returns self
#   For a file, returns the parent directory
#
# Input	:	PATH - path to be checked
# Output :	None
# Return Value:	PATH or result
#*******************************************************
#
proc tclu_get_dir_for_file {PATH} {
  if { [tclu_is_path_dir $PATH] } {
     return $PATH 
  } else {
     return [file dirname $PATH] 
  }
}

#*******************************************************
#
# Name :	tclu_get_bak_name
#
# Description :	
#   Returns a path which extends the input path with a suffix
#   of yyyymmdd and a serial number (If exists already)
#
# Input	:	PATH - path to be appended
# Output :	None
# Return Value:	New path
#*******************************************************
#
proc tclu_get_bak_name {PATH} {
   set DATE [clock format [clock seconds] -format "%Y%b%d"]   
   set NEW_PATH_ORG ${PATH}.${DATE}
   set NEW_PATH $NEW_PATH_ORG
   set COUNTER 0
   while { [tclu_file_exists $NEW_PATH] } {
      incr COUNTER
      set NEW_PATH ${NEW_PATH_ORG}.$COUNTER
   }
   return $NEW_PATH
}

#*******************************************************
#
# Name :	tclu_get_tmp_dir
#
# Description :	
#   Returns a path for temporary files
#
# Input	:	None
# Output :	None
# Return Value:	Path of temporary directory
#*******************************************************
#
proc tclu_get_tmp_dir {} {
   global TCLU_G
   global env

   if {$TCLU_G(OS) == "nt"} {
      return [tclu_path_back_to_fore $env(TEMP)]
   } else {
      return "/usr/tmp"
   }
}

#*******************************************************
#
# Name :	tclu_str_cmp_nc
#
# Description :	
#   Compare 2 strings (case insensitive)
#
# Input	:	S1, S2 - 2 strings
# Output :	None
# Return Value:	-1 for smaller, 0 for equal, 1 for larger
#*******************************************************
#
proc tclu_str_cmp_nc {S1 S2} {

   set S1_LC [string tolower $S1]
   set S2_LC [string tolower $S2]
   return [string compare $S1_LC $S2_LC]
}

#*******************************************************
#
# Name :	tclu_get_tmp_name
#
# Description :	
#   Returns a path  for a temporary file
#
# Input	:	PREFIX - A prefix for the file
# Output :	None
# Return Value:	New path
#*******************************************************
#
proc tclu_get_tmp_name {PREFIX} {
   return [format "%s/%s.%s" [tclu_get_tmp_dir] $PREFIX [pid]]
}

#*******************************************************
#
# Name :	tclu_copy_txt_file
#
# Description :	
#   Copies a text file, replacing lf to crlf if needed
#
# Input	:	IPATH - Input path
#               OPATH - Output path
# Output :	None
# Return Value:	None
#*******************************************************
#
proc tclu_copy_txt_file {IPATH OPATH } {

    if { [catch {open $IPATH RDONLY} IFP] } {
       tku_popup_message error ok $IFP
       return 1
    }
    if { [catch {open $OPATH { WRONLY CREAT }} OFP] } {
       close $IFP
       tku_popup_message error ok $OFP
       return 1
    }
    while {! [eof $IFP]} {
       gets $IFP LINE
       puts $OFP $LINE
    }
   
    close $IFP
    close $OFP
}

#*******************************************************
#
# Name :	tclu_del_tmp_files
#
# Description :	
#   Deletes all temp files of a process
#
# Input	:	PREFIX - A prefix for the file
# Output :	None
# Return Value:	New path
#*******************************************************
#
proc tclu_del_tmp_files {{PREFIX *}} {
   global TCLU_G
   global env

   if {$TCLU_G(OS) == "nt"} {
      set PATH [file join $env(TEMP) $PREFIX.[pid]]
   } else {
      set PATH [file join usr tmp $PREFIX.[pid]]
   }
   catch {file delete $PATH}
}

#*******************************************************
#
# Name :	tclu_check_files
#
# Description :	
#   Receives a root path and a list of relative paths and checks
#   against several lists of statis. A window is popped and shows all
#   results, marking in red the discrepancies. The user, in Custom mode
#   has the choice to overlook and continue.
#     
#
# Input	:	MODE       - Standard or Custom
#               ROOT_PATH  - Path of the root directory
#               PATH_LIST  - A list of relative paths, types and write/req
# Output :	
# Return Value:	New path
#*******************************************************
#
proc tclu_check_files {MODE ROOT_PATH PATH_LIST } { 
     
   set WLOGS \
     [tku_popup_log_open "Frontline Pro-Installer : Required Files Status" \
                     "-height 30"]
   set WLOG [lindex $WLOGS 0]
   set WT   [lindex $WLOGS 1]
   
   tku_popup_log $WT underline \
       "\nPath                                                      Exists  Type Rd Wr"
   tku_popup_log $WT normal "\n\n"
   set COUNT 0   
   foreach {PATH REQ_TYPE REQ_WRITABLE} $PATH_LIST {
      set FULL_PATH [file join $ROOT_PATH $PATH]
      set ASC_EXISTS    "No"
      set ASC_TYPE      "??"
      set ASC_READABLE  "??"
      set ASC_WRITABLE  "??"
      set STYLE "error"
      
      set EXISTS [tclu_file_exists $FULL_PATH]
      if { $EXISTS } {
         set ASC_EXISTS    "Yes"
	 set STYLE "normal"
	 if [tclu_is_path_file $FULL_PATH] {	    
	    set TYPE file 
	    set ASC_TYPE "File"
 	    set READABLE  [file readable $FULL_PATH]
	    if { $READABLE } {
	       set ASC_READABLE "Ok"   
	    } else {
	       set ASC_READABLE "No"
	    }
	 }
	 if [tclu_is_path_dir $FULL_PATH] {	    
	    set TYPE dir
	    set ASC_TYPE "Dir"
 	    set READABLE [expr \
	       [file readable $FULL_PATH] && [file executable $FULL_PATH] ]
	    if { $READABLE } {
	       set ASC_READABLE "Ok"   
	    } else {
	       set ASC_READABLE "No"
	    }
	 }	 
	 if { $TYPE != $REQ_TYPE } {
	    set STYLE "error"
	 }
	 if { ! $READABLE } {
	    set STYLE "error"
	 }
	 set WRITABLE [file writable $FULL_PATH]
	 if { $WRITABLE } {
	    set ASC_WRITABLE "Ok"   
	 } else {
	    set ASC_WRITABLE "No"
	 }
	    
	 if { ( $REQ_WRITABLE == "w" ) && ( ! $WRITABLE ) } {
	    set STYLE "error"
	 }	 
      }
      tku_popup_log $WT $STYLE \
	 [format "%-58s%-8s%-5s%-3s%-3s\n" \
              $FULL_PATH $ASC_EXISTS $ASC_TYPE $ASC_READABLE $ASC_WRITABLE] 
      incr COUNT
      if { [expr $COUNT % 5 ] == 0 } {
         tku_popup_log $WT normal "\n"
      }
	 
   }
   tku_popup_log_show $WLOG      
     
   return ok  
}

#*******************************************************
#
# Name :	tclu_is_root
#
# Description :	
#   Returns 1 if the caller has root privileges
#
# Input	:	None
# Output :	None
# Return Value:	1 for root, 0 for other
#*******************************************************
#
proc tclu_is_root {} {
   global TCLU_G
   global env
    
   if { $TCLU_G(IS_ROOT) == 1 } {
      return 1
   }
   if {$TCLU_G(OS) == "nt"} {
      if { [tclu_get_user] == "Administrator" } {
         return 1
      }
   } else {
      if { [tclu_get_user] == "root" } {
         return 1 
      }
   }
   return 0
}

#*******************************************************
proc tclu_set_root {IS_ROOT} {
   global TCLU_G
   set TCLU_G(IS_ROOT) $IS_ROOT
}
   

#*******************************************************
#
# Name :	tclu_file_exists
#
# Description :	
#   Returns 1 if input file exists.
#   Created because of Sun-Os Tcl problem - file exists
#   returns 1 when input name is ""
#
# Input	:	PATH
# Output :	
# Return Value:	1 for existing files, 0 for other
#*******************************************************
#
proc tclu_file_exists {PATH} {
  if {[string compare $PATH ""] == 0} {
    return 0
  }
  return [file exists $PATH]
}

#*******************************************************
#
# Name :	tclu_path_back_to_fore
#
# Description :	
# Convert all backslashes in path to forward slashes
#   Returns converted path
#
# Input	:	PATH
# Output :	none
# Return Value:	Path with foreward slashes
#*******************************************************
#
proc tclu_path_back_to_fore { PATH } {
  regsub -all "\\\\" $PATH "/" NEW_PATH
  return $NEW_PATH
}
#*******************************************************
#
# Name :	tclu_quote_backslashes
#
# Description :	
# Convert all backslashes in path to two backslashes for quoting 
# purposes
#
# Input	:	PATH
# Output :	none
# Return Value:	Path with double backslashes
#*******************************************************
#
proc tclu_quote_backslashes { PATH } {
  regsub -all "\\\\" $PATH "\\\\\\\\" NEW_PATH
  return $NEW_PATH
}

################ HOSTID UTITLITIES #######################

#*******************************************************
#
# Name :	tclu_get_hostid
#
# Description :	
#   Returns the 6 digits host id of the system, according to the given 
#   operating system.
#
# Input	:	OS - Operating system : aix, hp10, nt, solaris, linux
# Output :	None
# Return Value:	6 digit hostid
#*******************************************************
#
proc tclu_get_hostid {OS} {
   global TCLU_G

   switch -- $OS {
     aix {
       if { [catch {exec /bin/entstat ent0 | grep Hardware} HOSTLINE]  } {
	 # if 'ent0' did not work, we try tok0
	 if { [catch {exec /bin/tokstat tok0 | grep Hardware} HOSTLINE]  } {
	   tku_popup_message error ok \
	     "/bin/tokstat tok0 | grep Hardware :\n$HOSTLINE"
	   return ""
	 }
       }
       # Get 3rd word of line 
	 set HOSTWORD [lindex $HOSTLINE 2] 
       # Get 6 digits (every 2 are separated by dot) 
       set    HOSTID [string range $HOSTWORD 9 10]
       append HOSTID [string range $HOSTWORD 12 13]
       append HOSTID [string range $HOSTWORD 15 16]
       set HOSTID [string toupper $HOSTID]
     }
     hp10 {
         if { [catch {exec /etc/lanscan | grep 0x} HOSTLINE]  } {
	    tku_popup_message error ok \
	       "/etc/lanscan | grep 0x :\n$HOSTLINE"
	    return ""
	 }
         # Get 2nd word of line
	 set HOSTWORD [lindex $HOSTLINE 1] 
         # Get 6 digits 
	 set HOSTID   [string toupper [string range $HOSTWORD 8 13]]
     }
     nt {
        set EXEC [file join $TCLU_G(TOOLS) nodeid]
        if { [catch {exec $EXEC } HOSTLINE] } {
           tku_popup_message error ok \
              "$EXEC :\n$HOSTLINE"
           return ""
        }
        set HOSTID [string toupper [lindex $HOSTLINE 3]]
     }
     solaris {
        if { [catch {exec /usr/ucb/hostid} HOSTLINE]  } {
           tku_popup_message error ok \
              "/usr/ucb/hostid :\n$HOSTLINE"
           return ""
	}
        # Get 1st word of line
	set HOSTWORD $HOSTLINE
        # Get 6 digits 
	set HOSTID   [string toupper [string range $HOSTWORD 2 7]]
      }
      linux {
	 set HOSTID "XXXXXX"
      }
      default {
	 set HOSTID ""
      }
   }
   return $HOSTID    
}

#*******************************************************
#
# Name :	tclu_get_dongle
#
# Description :	
#   Returns the 16 digits dongle id of the system, by reading the
#   information from the dongle.
#
# Input	:	TOOLS - Path of proper TOOLS directory where dongle_info
#                       can be found
# Output :	None
# Return Value:	16 digit dongle id xxxx-xxxx-xxxx-xxxx
#*******************************************************
#
proc tclu_get_dongle {} {
   global TCLU_G
   
   set DONGLE_ID ""
   set DONGLE_INFO [file join $TCLU_G(TOOLS) dongle_info.exe]
     
   set TMP_PATH [tclu_get_tmp_name vpi_dongle]
   set TITLE "Searching for hardware key (dongle)..."
   set COMMAND [format "%s |& %s bs=1000000 of=%s" \
		     $DONGLE_INFO \
		     $TCLU_G(DD)  \
		     $TMP_PATH]
   set STATUS [tku_long_command $COMMAND $TITLE "" \
	 	    ignore ""]
   if { $STATUS != "ok" } {
      tku_popup_message error ok \
         [format "%s/n%s"                     \
         "Failed to retrieve the dongle id."  \
	 "Check log file for errors"]
      return ""
   }
		    
   if { [catch {open $TMP_PATH RDONLY} FP] } {
      tku_popup_message error ok \
	 "The command $DONGLE_INFO\
	\nproduced no output"
   } else {
      set DONGLE_DATA [read $FP]
      close $FP
	 
      foreach WORD $DONGLE_DATA {
	 if [string match ????-????-????-???? $WORD] {
	    set DONGLE_ID $WORD
	       break 
	 }
      }
   }
   if { [string length $DONGLE_ID] == 0 } {
      tku_popup_message error ok \
	 "The command $DONGLE_INFO\
	\nproduced the following errors :" $TMP_PATH
   }
   catch { file delete $TMP_PATH } 
   
   return $DONGLE_ID
}

#*******************************************************
#
# Name :	tclu_get_gz_size
#
# Description :	
#   Returns the size of a gzipped file
#
# Input	:	PATH - Path of the file to calculate
# Output :	None
# Return Value:	File size (or 0 if failed)
#*******************************************************
#
proc tclu_get_gz_size { PATH } {
  set SIZE 0

  if { ! [ catch {exec gunzip -l $PATH } RESULT ] } {
    set LIST [split $RESULT]
    set FNUM 0
    foreach FIELD $LIST {
      if { [string length $FIELD] == 0 } continue
      incr FNUM
      if { $FNUM == 6 } {
         set SIZE $FIELD
      }
    }
  }
  return $SIZE
}

################ LOG UTILITIES #########################
#
# The LOG utilities routines provide an easy mechanism to add lines
# or full files to a log file.
# Lines can be tagged in various ways to provide emphasis to some
# parts of the log file.
#
##################################
set TCLU_G(LOG_FP) 0

#*******************************************************
#
# Name :	tclu_log_set_path
#
# Description :	
#   Called once, by the main part of the application, setting the path
#   of the log file to be used.
#
# Input	:	PATH - Path of the log file to be used
# Output :	None
# Return Value:	None
#*******************************************************
#
proc tclu_log_set_path {PATH} {
  global TCLU_G
  set TCLU_G(LOG_PATH) $PATH
  return
}

proc tclu_log_get_path {} {
  global TCLU_G
  return $TCLU_G(LOG_PATH)
}

#*******************************************************
#
# Name :	tclu_log_newline
#
# Description :	
#   Adds a newline character to the log file (Opens it if necessary)
#
# Input	:	None
# Output :	None
# Return Value:	None
#*******************************************************
#
proc tclu_log_newline {} {
   global TCLU_G

   if [tclu.log_open] {return 1}
   puts $TCLU_G(LOG_FP) "" 
   return
}
   
#*******************************************************
#
# Name :	tclu_log_message
#
# Description :	
#   Adds a text message to the log file. The message can be multi line.
#   although the tag will only be active for the first line. The tag can be:
#   normal (default) - font: cbr14  color: black
#   error            - font: cbr14  color: red
#   underline        - font: cbr14  color: black  underline
#   title            - font: cbr18  color: black  underline
#   A new line is added by default after the last character.
# Input	:	MESSAGE - text message
#               TAG - One of the tags above (default = normal)
# Output :	None
# Return Value:	None
#*******************************************************
#
proc tclu_log_message {MESSAGE {TAG normal}} {
   global TCLU_G
   global env
      
   if [tclu.log_open] {return 1}
   set DATE [clock format [clock seconds] -format "%d %b %Y %H:%M:%S"]   
   if [string length $TAG] {
      puts -nonewline $TCLU_G(LOG_FP) "()$TAG "
   }
   puts $TCLU_G(LOG_FP) "$DATE [tclu_get_user]@[info hostname]: $MESSAGE"
   flush $TCLU_G(LOG_FP)
   return
}

#*******************************************************
#
# Name :	tclu_log_file
#
# Description :	
#   Adds a full file to the log file. Much faster then adding it line 
#   by line. No new line is added at the end of the file.
#
# Input	:	PATH - path to the file to add. 
# Output :	None
# Return Value:	None
#*******************************************************
#
proc tclu_log_file {PATH} {
   global TCLU_G

   if [tclu.log_open] {return 1}
   if {! [catch {open $PATH RDONLY} FP] } {
      set DATA [read $FP]
      close $FP
      puts -nonewline $TCLU_G(LOG_FP) \n$DATA
      flush $TCLU_G(LOG_FP)
   } else {
      tclu_log_message "Failed to add file $PATH" error
   }
   return
}

#*******************************************************
#
# Name :	tclu_create_lst_file
#
# Description :	
#   The routine gets a PATH, and creates an "lst" file in it.
#   as "lst file" is a text file that containes onr line for
#   each file in the input directory.
#   each line looks like this :
#   FNAME SIZE SUM
#   where FNAME is the file name
#         SIZE  is number of bytes in file
#         SUM   is md5sum of that file.
#   Note : If there was an "lst file" in the input directory,
#          it is deleted before processing.
#
# Input	:	PATH - files path
#               LST_NAME - name of lst file to create inside
#                          the input path.
# Output : None 
#*******************************************************
#
proc tclu_create_lst_file { LST_NAME PATH {PATH1 ""} } {
  global TCLU_G

  if { ! [tclu_is_path_dir $PATH] } {
    tku_popup_message error ok \
	"Directory $PATH does not exist, could not produce LST file"
    return 
  }     

  if { [string length $PATH1] > 0 } {
    if { ! [tclu_is_path_dir $PATH] } {
      tku_popup_message error ok \
	  "Directory $PATH does not exist, could not produce LST file"
      return 
    }     
  }

  set FULL_LST_NAME $LST_NAME.lst
  # just to be sure, delete the destination file first ...
  set LST_FILE_NAME [file join $PATH $FULL_LST_NAME]
  file delete $LST_FILE_NAME
  
  if { [string length $PATH1] == 0 } {
    set FLIST [glob [file join $PATH *]]
  } else {
    set FLIST [glob [file join $PATH *] [file join $PATH1 *] ]
  }
    
  if {[catch {open $LST_FILE_NAME w} FP]} {
    puts "Can't open $LST_FILE_NAME to produce LST file"
    return 
  }

  foreach FILE $FLIST {
    set SUM [tclu_get_checksum $FILE]
    set NAME [file tail $FILE]
    set SIZE [file size $FILE]
    set SUM_LINE [concat $NAME $SIZE $SUM]
    puts $FP $SUM_LINE
  }
  close $FP
}


#*******************************************************
#
# Name :	tclu_check_lst_file
#
# Description :	
#    The routine gets an lst file name and a directory
#    path. it then searches for the files inside the input
#    path, for matching parameters : name, size and sum.
#    The result is returned in a list, that each "element"
#    in it stands for: Name match (existance)
#                      Summatch   (by md5sum)
#                      Size       (in bytes)
#                      a "Should copy" flag
# Input	:	PATH - files path
#               LST_NAME - name of lst file to create inside
#                          the input path.
# Output :      LST_RESULT : a list, containing the result
#                            of lst matching.
#               
# Return Value : If all files match, returns 0, otherwise returns -1
#*****************************************************************
#
proc tclu_check_lst_file { PATH LST_FILE IGNORE_FILES LST {QUIET 0} } {
  global TCLU_G
  upvar $LST LST_RESULT
  
  set STATUS 0 
  
  if { ! [tclu_is_path_dir $PATH] } {
    if { !$QUIET } { 
      tku_popup_message error ok \
	  "Directory $PATH does not exist, could not check LST file"
    }
    set STATUS -1
    return $STATUS
  }     

  if { ! [tclu_file_exists $LST_FILE] } {
    if { !$QUIET } { 
      tku_popup_message error ok \
	  "Input LST file ($LST_FILE) does not exist, could not check..."
    }
    set STATUS -2
    return $STATUS
  }
  
  if { [catch {open $LST_FILE r} TFP]} {
    if { !$QUIET } { 
      tku_popup_message error ok \
	  "Error opening LST file ($LST_FILE), could not check..."
    }
    set STATUS -3
    return $STATUS
  }
  
  set FILE_COUNT 0     

  while {! [eof $TFP]} {
    gets $TFP LINE
    set LSIZE [string length $LINE]
    if {$LSIZE == 0} break  
    scan $LINE "%s%s%s" FILE_NAME FILE_SIZE FILE_SUM      
    set CUR_CHECK_FILE [file join $PATH $FILE_NAME]
    set FILE_NAMES($FILE_COUNT) $FILE_NAME
    set FILE_SIZES($FILE_COUNT) $FILE_SIZE
    set FILE_SUMS($FILE_COUNT) $FILE_SUM
    set FILE_COPY($FILE_COUNT) 1
    
    set NMATCH  "yes"
    set SUMATCH "yes"
    set COPYF   "no "

    if {[string match $IGNORE_FILES $FILE_NAME]} {
      continue
    }
    
    # if the current checked file exists, we make sure its
    # checksum is the same as we expect
    if { [tclu_file_exists $CUR_CHECK_FILE] } {
      set CUR_SUM [tclu_get_checksum $CUR_CHECK_FILE]
      if {[string compare $CUR_SUM $FILE_SUM] == 0} {
	set FILE_COPY($FILE_COUNT) 0
      } else {
	set STATUS -4
	set SUMATCH "no "
	set COPYF   "yes"
      }
    } else {
      set STATUS -4
      set NMATCH  "no "
      set SUMATCH "no "
      set COPYF   "yes"
    }
    
    lappend LST_RESULT $FILE_NAME $NMATCH $FILE_SIZE $SUMATCH $FILE_SUM $COPYF  
    incr FILE_COUNT      
  }
  close $TFP
  return $STATUS
}

#*******************************************************
#
# Name :	tclu_check_sum_file
#
# Description :	
#    The routine receives a data file and a sum file (in the standard
#    (name size md5sum) format and verifies that the md5sum of the 
#    file matches the sum file.
# Input	:	DATA_PATH - Path of the data file
#               SUM_PATH  - Path of the sum file
# Output :      None
#               
# Return Value : If file match, returns 0, otherwise returns -1
#*****************************************************************
#
proc tclu_check_sum_file { DATA_PATH SUM_PATH } {
  global TCLU_G
  
  set SUM_IS_OK 0
  if {! [catch {open $SUM_PATH r} TFP]} {
    gets $TFP LINE
    close $TFP
    set NUM_FIELDS [scan $LINE "%s %s %s" FILE_NAME SIZE SUM]
    if {$NUM_FIELDS != 3} {
       return -1 
    }
    if { [tclu_file_exists $DATA_PATH] } {
      set CUR_SUM [tclu_get_checksum $DATA_PATH]
      set CUR_SIZE [file size $DATA_PATH]
      if {($CUR_SUM == $SUM) && ($CUR_SIZE == $SIZE)} {
	set SUM_IS_OK 1 
      }
    }
  } 
  if { ! $SUM_IS_OK } {
    return -1 
  }
  
  return 0
}

proc tclu_get_checksum { IN_FILE } {   

  global TCLU_G

  set SUM_PATH [tclu_get_tmp_name "tmpsum"]
  set SUMPROG [file join $TCLU_G(TOOLS) "md5sum.exe"]
  set SUM 0 
  if { ! [tclu_file_exists $IN_FILE] } {
    tku_popup_message error ok \
	"$IN_FILE not found - checksum set to 0"
    return $SUM
  }
  set PIPE_IN [file join $TCLU_G(TOOLS) pipe_in.exe]
  set PROGRESS_PATH [tclu_get_tmp_name "progress"]
  set COMMAND [format "%s %s %s | %s -b | %s bs=1000000 of=%s" \
                   $PIPE_IN $IN_FILE $PROGRESS_PATH \
		   $SUMPROG \
		   $TCLU_G(DD)     \
		   $SUM_PATH]
  set TITLE "Calculating checksum for $IN_FILE" 
  set STATUS [tku_long_command $COMMAND \
		  $TITLE   \
		  "" \
		  ignore $PROGRESS_PATH]		 
  catch { file delete $PROGRESS_PATH } 
  if { $STATUS != "ok" } {
    return 0
  }
  
  if {! [catch {open $SUM_PATH r} TFP]} {
    gets $TFP LINE
    close $TFP
    catch { file delete $SUM_PATH } 
    scan $LINE "%s%s" SUM NAME
  } else {
    tku_popup_message error ok \
	"Error Opening $SUM_PATH - checksum set to 0"
    set SUM 0 
  }
  return $SUM 
}


#*******************************************************
#
# Name :	tclu_get_dat_lst_file
#
# Description :	
#               The routine will read from the dat device the
#               products.lst file, and will return the items
#               on dat as a list.
# Input	:	DAT_DEVICE - Path of dat device
#
# Output :      DAT_LIST - List of available procucts on dat
#                            
#               
# Return Value :
#*****************************************************************
#
#................................................................#
proc tclu_get_dat_lst_file {DAT_DEVICE OUT_LIST} {

  global TCLU_G  
  upvar $OUT_LIST DAT_LIST
  
  set SAVDIR [pwd]
  cd [tclu_get_tmp_dir]
  
  set TAR $TCLU_G(TAR)
  set COMMAND "$TAR xf $DAT_DEVICE products.lst end_products.lst"
  set TITLE "Extracting products list from DAT..."
  set STOP_FILE [file join [tclu_get_tmp_dir] "end_products.lst"]

  file delete [file join [tclu_get_tmp_dir] "products.lst"]
  file delete $STOP_FILE
  
  set STATUS [tku_long_command $COMMAND $TITLE "" abort "" $STOP_FILE]		 
  
  if { $STATUS != "ok" } {
    return $STATUS
  }
  
  cd $SAVDIR
  set LST_NAME [file join [tclu_get_tmp_dir] "products.lst"]
  if {[catch {open $LST_NAME r} FP]} {
    return "error"
  }

  catch {unset DAT_LIST}
  while {![eof $FP]} {
    gets $FP LINE
    if { [string length $LINE] == 0 } break ; 
    scan $LINE "%s%s%s%s%s%s%s%s" DUM1 PROD DUM2 VER DUM3 OS DUM4 TITLE 
    set INDEX [string first "title:" $LINE]
    if { $INDEX == -1 } {
      set TITLE "<No Title>"
    } else {
      set START [expr $INDEX + 7]
      set TITLE [string range $LINE $START end]
    }
    
    lappend DAT_LIST $PROD $VER $OS $TITLE
  }

  file delete [file join [tclu_get_tmp_dir] "products.lst"]
  return "ok"
}


#*******************************************************
#
# Name :	tclu_update_file
#
# Description : updates the input file according the input parameters.
# Input	:	IN_FILE - file to edit
#               IN_TAG  - tag to search in file. could contain "*" in it.
#               VAR     - serial number of value in the expected line,
#                         begining with 1.
#                         for exampl, if we wanted to change the next line
#                         maxdsiz 111111 -> To
#                         maxdsiz 222222
#                         the VAR to supply must be 2.
#               NEW_VAL - the new value to set
#               PUT_NEW_VAL - Add value if not found
#               REMARK  - the IN_FILE remark sign
#               DELIM   - delimiter sign, if exists.
#               VAL_TYPE - one of "numeric" or "str", indicates the type
#                          of input parameter.
# Output :      "error" or "ok"
#               
# Note : for "numeric" values, they will be changed only of the input
#        NEW_VAL is greater than the current one.
#        for "str" values, they will be changed when they are different
#        from the current value.
#*****************************************************************
proc tclu_update_file { IN_FILE IN_TAG VAR NEW_VAL \
			    {PUT_NEW_VAL 1} {REMARK \#}  \
			    { DELIM "" } {VAL_TYPE numeric }}  {
			 
  set TMP_FILE [tclu_get_tmp_name tplugin]
  if {[catch {open $IN_FILE r} FP]} {
    tku_popup_message error ok $FP
    return "error"
  }

  if {[catch {open $TMP_FILE w} TFP]} {
    tku_popup_message error ok $TFP
    return "error"
  }
  
  set FOUND 0 
  set TAG $IN_TAG*
  
  while {![eof $FP]} {
    gets $FP LINE
    if { [string length $LINE] == 0 } {
      puts $TFP ""
      continue
    }    
    set MATCH [string match $TAG $LINE]
    if {$MATCH} {
      if { [string range $LINE 0 0] != $REMARK } {
	set DO_BREAK 1
	set FOUND 1
	set F1 ""
	set F2 ""
	set F3 ""
	set F4 ""
	set F5 ""
	set F6 ""
	set COUNT [scan $LINE "%s%s%s%s%s%s" F1 F2 F3 F4 F5 F6]
	if {$COUNT < $VAR} {
	  return "ok"
	}

	if { $DELIM == "" } {
	  set FNAME [set F$VAR]
	  set TEST_VAL $FNAME
	} else {
	  set IND [string first $DELIM $LINE]
	  if { $IND == -1 } {
	    set TEST_VAL 0
	  } else {
	    incr IND
	    set TEST_VAL [string range $LINE $IND end]
	  }
	}
	if { $VAL_TYPE == "numeric" } {
	  set CHANGE {$NEW_VAL > $TEST_VAL}
	} else {
	  if { $NEW_VAL == "" } {
	    set CHANGE 0
	  } else { 
	    set CHANGE [string compare $NEW_VAL $TEST_VAL]
	  }
	}
	if { $CHANGE != 0 } {
	  if { $DELIM == "" } {	  
	    set F$VAR $NEW_VAL
	    puts $TFP "$F1 $F2 $F3 $F4 $F5 $F6"
	  } else {
	    incr IND -1
	    puts -nonewline $TFP [string range $LINE 0 $IND]
	    puts $TFP $NEW_VAL
	  }
	} else {
	  puts $TFP $LINE
	}
      } else {
	puts $TFP $LINE
	set DO_BREAK 0
      }
      if { $DO_BREAK } {
	break
      }
    } else {
      puts $TFP $LINE
    }
  }
  
  if {$FOUND} {
    while {![eof $FP]} {
      gets $FP LINE
      if { [string trim $LINE] == "" } continue
      puts $TFP $LINE
    }
  } else {
    regsub -all {\*} $IN_TAG " " IN_TAG 
    if { $PUT_NEW_VAL } {
      puts $TFP "$IN_TAG $NEW_VAL"
    } else {
      puts $TFP "$IN_TAG"
    }
  }
  
  close $TFP
  close $FP

  if [catch {file copy -force $TMP_FILE $IN_FILE} RESPONSE] {
    tku_popup_message error ok $RESPONSE
    tclu_del_tmp_files tplugin
    return "error"
  }	 
  tclu_del_tmp_files tplugin
  return "ok"
}

#*******************************************************
#
# Name :	tclu_find_str_in_file
#
# Description : The routine searches the IN_FILE for the IN_TAG
#               IN_TAG string. when founds, it adds after
#               string
#               lines begining with the REMARK sign are discarded.
# Input	:	IN_FILE - file to search
#               IN_TAG  - string to search inside the input file
# Output :     -1 - unexpected error
#               0 - string wasn't found
#               1 - string was found
#               
# Return Value : If file match, returns 0, otherwise returns -1
#*****************************************************************
proc tclu_find_str_in_file { IN_FILE IN_TAG {REMARK "\#"} {LIMIT 0} } {
  
  set STATUS 0
  set TAG $IN_TAG*

  if {[catch {open $IN_FILE r} FP]} {
    tku_popup_message error ok $FP
    return -1
  }

  set LCOUNT 0
  while {![eof $FP]} {
    gets $FP LINE
    incr LCOUNT
    if { [string length $LINE] == 0 } continue 
    if { [string range $LINE 0 0] == $REMARK } continue
    set MATCH [string match $TAG $LINE]
    if {$MATCH} {
      set STATUS 1
      break ; 
    }
    if { $LIMIT <= 0 } continue
    if { $LCOUNT > $LIMIT } break 
  }
  close $FP
  
  return $STATUS
}

#*******************************************************
#
# Name :	tclu_put_after
#
# Description : The routine searches the IN_FILE for the
#               IN_TAG string. when founds, it adds after
#               it the VALUE line.
#               lines begining with the REMARK sign are discarded.
#               if IN_TAG wasn't found, nothing is changed
#               in  the IN_FILE file.
# Input	:	IN_FILE - file to search
#               IN_TAG  - string to search inside the input file
#               VALUE   - string to add after the IN_TAG string,
#                         if found
#               REMARK  - remark sign, to ignore lines begining with.
# Output :      "error" or "ok"
#               
# Return Value : If file match, returns 0, otherwise returns -1
#*****************************************************************
#######################################################

proc tclu_put_after { IN_FILE IN_TAG VALUE { REMARK "\#"} } {
  
  set STATUS "error"
  set TAG $IN_TAG*

  set TMP_FILE [tclu_get_tmp_name tplugin]
  if {[catch {open $TMP_FILE w} TFP]} {
    tku_popup_message error ok $TFP
    return "error"
  }

  if {[catch {open $IN_FILE r} FP]} {
    tku_popup_message error ok $FP
    return "error"
  }

  set BLIND 0 
  while {![eof $FP]} {
    gets $FP LINE
    if { [string length $LINE] == 0 } {
      puts $TFP ""
      continue 
    }
    if { $BLIND} {
      puts $TFP $LINE
      continue
    }
    if { [string range $LINE 0 0] == $REMARK }  {
      puts $TFP $LINE
      continue
    }
    set MATCH [string match $TAG $LINE]
    if {$MATCH} {
      puts $TFP $LINE
      puts $TFP $VALUE
      set BLIND 1 
      continue
    } else {
      puts $TFP $LINE
      continue
    }
  }
  
  close $FP
  close $TFP

  file copy -force $TMP_FILE $IN_FILE
  tclu_del_tmp_files tplugin

  return "ok"
}

#*******************************************************
#
# Name :	tclu_put_at_eof
#
# Description : The routine adds at the end of the file the input
#               IN_VALUE
# Input	:	IN_FILE - file to edit
#               VALUE   - string to add at the end of the file
# Output :      "error" or "ok"
#               
# Return Value : If file match, returns 0, otherwise returns -1
#*****************************************************************
proc tclu_put_at_eof { IN_FILE IN_VALUE } {
  
  set STATUS "error"

  set TMP_FILE [tclu_get_tmp_name tplugin]
  if {[catch {open $TMP_FILE w} TFP]} {
    tku_popup_message error ok \
 	"Cannot open file $TMP_FILE To edit $IN_FILE"
    return "error"
  }

  if {[catch {open $IN_FILE r} FP]} {
    tku_popup_message error ok \
 	"Cannot open file $IN_FILE to edit"
    return "error"
  }

  while {![eof $FP]} {
    gets $FP LINE
    puts $TFP $LINE
  }
  
  puts $TFP $IN_VALUE
  close $FP
  close $TFP
  
  file copy -force $TMP_FILE $IN_FILE
  tclu_del_tmp_files tplugin

  return "ok"
}

#******** Private Procedure ***************************
#
# Name :	tclu.log_open
#
# Description :	
#   Make sure the log file is open. If not, creates it with full
#   open privileges. If the file exists, seeks to the end so information
#   will be appended. The PATH is taken from the global var set by 
#   tclu_log_set_path.
# Input	:	None
# Output :	None
# Return Value:	1 if error
#               0 if success
#*******************************************************
#
proc tclu.log_open {} {   
   global TCLU_G
      
   if {$TCLU_G(LOG_FP) == -1} {return 1}
   if {$TCLU_G(LOG_FP) == 0} {
      if [catch {open $TCLU_G(LOG_PATH) {RDWR CREAT} 0777} TCLU_G(LOG_FP)] { 
	 puts "Cannot open log file $TCLU_G(LOG_PATH)"
	 set TCLU_G(LOG_FP) -1 
	 return 1
      } else {
	 seek $TCLU_G(LOG_FP) 0 end
      }
   }
   return 0 
}

#*******************************************************
#
# Name :	tclu_set_system_env
#
# Description : The routine sets a system-wide environment variable
#               permanently!
#
# Note        : Currently implemented for Windows NT only!
#               
# Input	:	VARIABLE - the name of the environment variable
#               VALUE    - the value of the environment variable
# Output :      none
#               
# Return Value : "error" or "ok"
#*****************************************************************
proc tclu_set_system_env {VARIABLE VALUE} {

#   set ROOT "HKEY_LOCAL_MACHINE"
#   set HOST [hostname]
#   set ENV  "\\\\$HOST\\$ROOT\\SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment"
#   registry set $ENV $VARIABLE $VALUE sz
   global TCLU_G
   global env

   set PATH "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment"

   set EXEC [file join $TCLU_G(TOOLS) reg_set.exe]
   if { [catch {exec $EXEC $PATH $VARIABLE REG_SZ $VALUE } HOSTLINE] } {
      tku_popup_message error ok \
         "$EXEC :\n$HOSTLINE"
      return error
   }
   set env($VARIABLE) $VALUE
   return ok
}

#*******************************************************
#
# Name :	tclu_null_file
#
# Description : The routine returns the path of a null file
#
# Input	:	none
#               
# Return Value : path of null file
#                (on Unix "/dev/null", on NT "nul")
#*****************************************************************
proc tclu_null_file {} {

   global TCLU_G
   if { $TCLU_G(OS) == "nt" } {
#      return "c:/tmp/null"
      return "nul"
   } else {
      return "/dev/null"
   }
}

