/***********************************************************
 *
 *  Module: imp.h
 *
 *  Description:
 *
 *  This module contains literals, macros and structures which are required
 *  for impedance calculations
 *
 *  Author:		Ben Michelson
 *
 *  (c) Copyright 1996 - Valor Computerized Systems Ltd.
 * 
 *
 * Naming conventions:
 *
 *    IMP_M_*  -- macros
 *    IMP_L_*  -- literals
 *
 * Revision: 2.0 30 Oct 96
 *           Impedance models changed
 *           model added to imp_out
 ***********************************************************/
#include <stdio.h>
#include <math.h>
#define IMP_L_ERROR 1
#define IMP_L_OK    0
#define IMP_STATUS_CHECK \
   { if (status != IMP_L_OK) { \
      fprintf(stderr,"Impedance program error in file %s line %d\n", \
              __FILE__,__LINE__); \
      return(status); }}
#define IMP_ASSERT(x) { if (!(x)) { \
   printf("(" #x ") at %s %d\n",__FILE__,__LINE__); \
   return (IMP_L_ERROR);}}

#define IMP_L_LEN_NAME       32
#define IMP_L_LEN_LAYER_NAME 64
#define IMP_L_CONSTRUCT_LEN  15

typedef enum {
   IMP_L_NULL_MODEL = -1,
   IMP_L_MSTRIP_SURFACE,
   IMP_L_MSTRIP_EMBEDDED,
   IMP_L_SLINE,
   IMP_L_SLINE_DUAL,
   IMP_L_DIFF_BROAD,
   IMP_L_DIFF_MSTRIP_SURFACE,
   IMP_L_DIFF_MSTRIP_EMBEDDED,
   IMP_L_DIFF_SLINE,
   IMP_L_NUM_MODEL } imp_model_enum;

#define IMP_M_LEGAL_MODEL(m) ((m) >= 0 && (m) < IMP_L_NUM_MODEL)

typedef enum { IMP_L_NULL_WEAVE, 
               IMP_L_NO_WEAVE,
               IMP_L_VERT_WEAVE,
	       IMP_L_HORIZ_WEAVE,
	       IMP_L_NUM_WEAVE } imp_weave_enum; /* Weave of dielectric */

typedef enum { IMP_L_NULL_MATERIAL,
               IMP_L_FOIL_MATERIAL,
	       IMP_L_LAM_MATERIAL,
	       IMP_L_PREPREG_MATERIAL,
	       IMP_L_RESIN_MATERIAL,
	       IMP_L_SPARE2_MATERIAL,
	       IMP_L_NUM_MATERIAL_TYPE } imp_material_enum;

typedef enum { IMP_L_STD_DUCTILITY,
               IMP_L_HTE_DUCTILITY,
               IMP_L_NUM_DUCTILITY } imp_ductility_enum; /* for foil */

typedef struct {
   float thick;                   /* Thickness in mils                   */
   float pos_tol;                 /* Tolerance of thickness +            */
   float neg_tol;                 /* Tolerance of thickness -            */
} imp_thick_struc;                /* for all materials                   */

typedef struct {
   imp_ductility_enum ductility;  /* From `material' file                */
   double             resistance; /* This is currently always 0          */
   char               layer_name[IMP_L_LEN_LAYER_NAME + 1];
   char               spare[200]; /* Unused                              */
} imp_foil_struc;

typedef struct {
   imp_weave_enum     weave;      /* From `bill' file                    */
   double             dry_Er;     /* From `material' file                */
   float              resin_percent;     
                                  /* From `material' file                */
   double             dry_weight; /* From `material' file                */
   char               spare[200]; /* Unused                              */
} imp_substrate_struc;   

typedef struct {
   char              name[IMP_L_LEN_NAME+1];    
   char              vendor[IMP_L_LEN_NAME+1];
   char              cat_num[IMP_L_LEN_NAME+1];
   imp_thick_struc   thick;       /* in mils                             */
   int               internal;    /* for internal use                    */
   imp_material_enum material;
   union {
      imp_foil_struc      foil;
      imp_substrate_struc substrate;
   } u;
   char               spare[200]; /* Unused                              */
} imp_material_struc;

typedef struct {
   float              width;      /* from `Stackup parameters' window    */
   float              height;     /* from `Stackup parameters' window    */
   char               construct[IMP_L_CONSTRUCT_LEN+1];
   double             resin_Er;   /* From header of `bill' file          */
   double             etch_factor;/* from impedance                      */
                                  /* `Initialization Parameters' window  */
   float              plate_thick;/* Plating thickness                   */
                                  /* from `Stackup parameters' window    */
   float              mask_thick; /* SS & SM thickness                   */
                                  /* from `Stackup parameters' window    */
   double             mask_Er;    /* from impedance                      */
                                  /* `Initialization Parameters' window  */
   imp_model_enum     model;      /* From matrix                         */
   imp_thick_struc    line_width; /* From matrix                         */
   double             spacing;    /* In mils -- for differential models  */
   imp_thick_struc    thick;      /* Thickness of the whole board,       */
                                  /* metal to metal (without plating)    */
   
   double             freq;       /* from impedance                      */
                                  /* `Initialization Parameters' window  */
   
   imp_material_struc *material;  /* All Materials of stackup            */ 
   int                num_material; 
                                  /* Number of materials in stackup      */   
   int                alloc_material; 
                                  /* For internal use                    */
   int                ref1;       /* Index of first reference plane      */
                                  /* in `material' */
   int                ref2;       /* Index of first reference plane      */
                                  /* in `material' (for stripline)       */
   int                lyr;        /* Index of signal layer in `material' */
   int                dual_lyr;   /* Index of dual signal layer in       */
                                  /* `material'-for differential stripline*/
   imp_thick_struc    dielectric_h[3];
                                  /* Contains up to 3 dielectric         */
                                  /* thicknesses                         */
   double             Er;         /* The average relative permittivity,  */
                                  /* calculated as a weighted average    */
   unsigned char      fixed_model;/* May model choose submodel?          */          
   char               spare[200]; /* unused                              */
} imp_in_struc;

typedef struct{
   double ohm;
   double pos_tol;                /* In ohms                              */
   double neg_tol;                /* In ohms                              */
} imp_ohm_struc;
   
typedef struct {
   imp_ohm_struc  ohm;
   imp_model_enum model; 
   char           spare[200];     /* unused                               */
} imp_out_struc;

#define IMP_M_SHEET_WIDTH(imp)     (imp)->width
#define IMP_M_SHEET_HEIGHT(imp)    (imp)->height
#define IMP_M_CONSTRUCT(imp)       (imp)->construct
#define IMP_M_RESIN_Er(imp)        (imp)->resin_Er
#define IMP_M_ETCH_FACTOR(imp)     (imp)->etch_factor
#define IMP_M_PLATE_THICK(imp)     (imp)->plate_thick
#define IMP_M_MASK_THICK(imp)      (imp)->mask_thick
#define IMP_M_MASK_Er(imp)         (imp)->mask_Er
#define IMP_M_MODEL(imp)           (imp)->model
#define IMP_M_W(imp)               (imp)->line_width
#define IMP_M_SPACING(imp)         (imp)->spacing
#define IMP_M_FREQ(imp)            (imp)->freq
#define IMP_M_T(imp)               (imp)->thick
#define IMP_M_Er(imp)              (imp)->Er
#define IMP_M_SPACING(imp)         (imp)->spacing

#define IMP_M_MSTRIP_H(imp)        (imp)->dielectric_h[0]
#define IMP_M_MSTRIP_Er(imp)       IMP_M_Er(imp)
#define IMP_M_MSTRIP_T(imp)        IMP_M_T(imp)
#define IMP_M_MSTRIP_W(imp)        IMP_M_W(imp)
#define IMP_M_MSTRIP_H2(imp)       (imp)->dielectric_h[1]

#define IMP_M_SLINE_H1(imp)        (imp)->dielectric_h[0]
#define IMP_M_SLINE_H2(imp)        (imp)->dielectric_h[1]
#define IMP_M_SLINE_T(imp)         IMP_M_T(imp)
#define IMP_M_SLINE_W(imp)         IMP_M_W(imp)
#define IMP_M_SLINE_Er(imp)        IMP_M_Er(imp)

#define IMP_M_DIFF_MSTRIP_Er(imp)  IMP_M_Er(imp)
#define IMP_M_DIFF_MSTRIP_T(imp)   IMP_M_T(imp)
#define IMP_M_DIFF_MSTRIP_W(imp)   IMP_M_W(imp)
#define IMP_M_DIFF_MSTRIP_H(imp)   (imp)->dielectric_h[0]
#define IMP_M_DIFF_MSTRIP_H2(imp)  (imp)->dielectric_h[1]

#define IMP_M_DIFF_SLINE_H1(imp)   (imp)->dielectric_h[0]
#define IMP_M_DIFF_SLINE_H2(imp)   (imp)->dielectric_h[1]
#define IMP_M_DIFF_SLINE_T(imp)    IMP_M_T(imp)
#define IMP_M_DIFF_SLINE_W(imp)    IMP_M_W(imp)
#define IMP_M_DIFF_SLINE_Er(imp)   IMP_M_Er(imp)

#define IMP_M_DIFF_BROAD_H1(imp)   (imp)->dielectric_h[0]
#define IMP_M_DIFF_BROAD_H2(imp)   (imp)->dielectric_h[1]
#define IMP_M_DIFF_BROAD_H3(imp)   (imp)->dielectric_h[2]
#define IMP_M_DIFF_BROAD_T(imp)    IMP_M_T(imp)
#define IMP_M_DIFF_BROAD_W(imp)    IMP_M_W(imp)
#define IMP_M_DIFF_BROAD_Er(imp)   IMP_M_Er(imp)


