#include "imp.h"
#include <math.h>
#define DEBUGx
/*
 *
 *                      Impedance Functions
 *
 * Thanks to Bill Reeves of AMP Circuits for functions for
 * differential impedance
 *
 * Coded by Ben Michelson
 * Nov 96
 *
 * Notes:
 *   * For microstrip, it is considered surface, if there is less
 *     than 1 mil of dielectric coating.
 *
 */

static int imp_lib_mstrip(imp_in_struc *imp_in, imp_out_struc *imp_out_p);
static int imp_lib_sline(imp_in_struc *imp_in, imp_out_struc *imp_out_p);
static int imp_lib_diff_broad(imp_in_struc *imp_in, imp_out_struc *imp_out_p);
static int imp_lib_diff_mstrip(imp_in_struc *imp_in, imp_out_struc *imp_out_p);
static int imp_lib_diff_sline(imp_in_struc *imp_in, imp_out_struc *imp_out_p);

/*PR*/
/*PE*/
/*PT*/
int imp_lib_calc(imp_in_struc  *imp_in, 
		 imp_out_struc *imp_out_p)
{
   int status;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);
   
   if (IMP_M_MSTRIP_Er(imp_in) <= 0.0) { /* patch to avoid crashing on */
      IMP_M_MSTRIP_Er(imp_in) = 5.0;     /* undefined Er               */
   }
   
   switch (IMP_M_MODEL(imp_in)) {
      case IMP_L_MSTRIP_SURFACE:
      case IMP_L_MSTRIP_EMBEDDED:
	 status = imp_lib_mstrip(imp_in, imp_out_p);
	 IMP_STATUS_CHECK;
	 break;
      case IMP_L_SLINE:
      case IMP_L_SLINE_DUAL:
	 status = imp_lib_sline(imp_in, imp_out_p);
	 IMP_STATUS_CHECK;
	 break;
      case IMP_L_DIFF_BROAD:
	 status = imp_lib_diff_broad(imp_in, imp_out_p);
	 IMP_STATUS_CHECK;
	 break;
      case IMP_L_DIFF_MSTRIP_SURFACE:
      case IMP_L_DIFF_MSTRIP_EMBEDDED:
	 status = imp_lib_diff_mstrip(imp_in, imp_out_p);
	 IMP_STATUS_CHECK;
	 break;
      case IMP_L_DIFF_SLINE:
	 status = imp_lib_diff_sline(imp_in, imp_out_p);
	 IMP_STATUS_CHECK;
	 break;
      default:
	 IMP_ASSERT (IMP_M_MODEL(imp_in) && 0);
   }
   return (IMP_L_OK);
}

/* formula from ipc-d-317 */
static int imp_lib_mstrip(imp_in_struc *imp_in, imp_out_struc *imp_out_p)
{
   imp_ohm_struc Z0;
   double        H;
   double        H2;
   double        t; /* line thickness       */
   double        w; /* line width           */
   double        Er;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);

   H  = IMP_M_MSTRIP_H(imp_in).thick;
   Er = IMP_M_MSTRIP_Er(imp_in);
   t  = IMP_M_MSTRIP_T(imp_in).thick;
   w  = IMP_M_MSTRIP_W(imp_in).thick;
   H2 = IMP_M_MSTRIP_H2(imp_in).thick;
      
   if (H2 == 0.0) {
      /* surface microstrip */
      ;
   }
   else {
      /* embedded microstrip */
      Er = Er * (1 - exp(-1.55 * (H2 + H + t) / H));
   }
   Z0.ohm = (87.0 * log(5.98 * H / (0.8 * w + t)) / sqrt(Er + 1.41));
   Z0.pos_tol = Z0.ohm * 0.05;
   Z0.neg_tol = Z0.ohm * 0.05;

#ifdef DEBUG
   if (H2 == 0.0) {
      printf("Surface_Microstrip\n"
	     "\t(H=%lg Er=%lg t=%lg w=%lg, H2=%lg) = %lg\n",
	     H, Er, t, w, H2, Z0.ohm);
   } else {
      printf("Embedded_Microstrip\n"
	     "\t(H=%lg Er=%lg t=%lg w=%lg, H2=%lg) = %lg\n",
	     H, Er, t, w, H2, Z0.ohm);
   }
#endif
   
   imp_out_p->ohm = Z0;
   return (IMP_L_OK);
}

/* formula from ipc-d-317 for symmetric stripline */
static int imp_lib_sline(imp_in_struc *imp_in, imp_out_struc *imp_out_p)
{
   imp_ohm_struc Z0;
   double        H1;
   double        H2;
   double        t; /* line thickness       */
   double        w; /* line width           */
   double        Er;
   double        H_avg;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);

   H1 = IMP_M_SLINE_H1(imp_in).thick;
   H2 = IMP_M_SLINE_H2(imp_in).thick;
   Er = IMP_M_SLINE_Er(imp_in);
   t  = IMP_M_SLINE_T(imp_in).thick;
   w  = IMP_M_SLINE_W(imp_in).thick;
   
   if (H1 == H2) {
      /* symmetric stripline */
      Z0.ohm = 60.0 * log(1.9 * (2.0 * H1  + t)/(0.8 * w + t)) / sqrt(Er);
      Z0.pos_tol = Z0.ohm * 0.05;
      Z0.neg_tol = Z0.ohm * 0.05;
#ifdef DEBUG
      printf("Stripline\n"
	     "\t(H=%lg Er=%lg t=%lg w=%lg) = %lg\n",
	     H1, Er, t, w, Z0.ohm);
#endif
   }
   else {
      /* asymmetric stripline */
      H_avg = (H1 + H2) / 2.0;
      Z0.ohm = 60.0 * log(1.9 * (2.0 * H_avg  + t)/(0.8 * w + t)) / sqrt(Er);
      Z0.pos_tol = Z0.ohm * 0.10;
      Z0.neg_tol = Z0.ohm * 0.10;
#ifdef DEBUG
      printf("Dual Stripline\n"
	     "\t(H1=%lg H2=%lg Er=%lg t=%lg w=%lg, ef=%lg) = %lg\n",
	     H1, H2, Er, t, w, ef, Z0.ohm);
#endif
   }
   imp_out_p->ohm = Z0;
   return (IMP_L_OK);
}

static int imp_lib_diff_broad(imp_in_struc *imp_in, imp_out_struc *imp_out_p)
{
   imp_ohm_struc Z0;
   double        H1;
   double        H2;
   double        H3;
   double        t; /* line thickness       */
   double        w; /* line width           */
   double        Er;
   double        H_avg;
   double        c;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);

   H1 = IMP_M_DIFF_BROAD_H1(imp_in).thick;
   H2 = IMP_M_DIFF_BROAD_H2(imp_in).thick;
   H3 = IMP_M_DIFF_BROAD_H3(imp_in).thick;
   Er = IMP_M_DIFF_BROAD_Er(imp_in);
   t  = IMP_M_DIFF_BROAD_T(imp_in).thick;
   w  = IMP_M_DIFF_BROAD_W(imp_in).thick;
   c  = IMP_M_SPACING(imp_in);

   memset(imp_out_p, 0, sizeof(*imp_out_p));

   /* formula from ipc-d-317 for symmetric dual stripline */
   /*
   if (H1 == H2) {
      Z0.ohm = 80 * log(1.9 * (2.0 * H1 + t) / (0.8 * w + t)) 
	 * (1 - H1 / (4 * (H1 + H2 + t))) / sqrt(Er);
      Z0.pos_tol = Z0.ohm * 0.05;
      Z0.neg_tol = Z0.ohm * 0.05;
   }
   else {
      H_avg = (H1 + H3) / 2.0;
      Z0.ohm = 80.0 * log(1.9 * (2.0 * H_avg + t) / (0.8 * w + t)) 
	 * (1 - H_avg / (4.0 * (H_avg + H2 + t))) / sqrt(Er);
      Z0.pos_tol = Z0.ohm * 0.10;
      Z0.neg_tol = Z0.ohm * 0.10;
   }
   imp_out_p->ohm = Z0;
   */
   return (IMP_L_OK);
}

/* Courtesy of Bill Reeves, AMP */
static int imp_lib_diff_mstrip(imp_in_struc *imp_in, imp_out_struc *imp_out_p)
{
   imp_ohm_struc Z0;
   double        H1;
   double        H2;
   double        t; /* line thickness       */
   double        w; /* line width           */
   double        Er;
   double        c;
   double        K;
   double        w1;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);

   H1 = IMP_M_DIFF_MSTRIP_H(imp_in).thick;
   H2 = IMP_M_DIFF_MSTRIP_H2(imp_in).thick;
   Er = IMP_M_DIFF_MSTRIP_Er(imp_in);
   t  = IMP_M_DIFF_MSTRIP_T(imp_in).thick;
   w  = IMP_M_DIFF_MSTRIP_W(imp_in).thick;
   c  = IMP_M_SPACING(imp_in);

   memset(imp_out_p, 0, sizeof(*imp_out_p));
   if (c < 0.01) {
      return (IMP_L_OK);
   }
   w1 = w * (w / c);

   if (H2 < 2.0) { /* Emdedded */
      K = 87.0;
      imp_out_p->model = IMP_L_DIFF_MSTRIP_SURFACE;
   } else if (H2 < 20.0) {
      K = 65.0;
      imp_out_p->model = IMP_L_DIFF_MSTRIP_EMBEDDED;
   } else {
      K = 60.0;
      imp_out_p->model = IMP_L_DIFF_MSTRIP_EMBEDDED;
   }

   Z0.ohm = (K / (sqrt(Er + 1.41)) * (log(5.98 * H1 / (0.8 * w1 + t)))) * 1.64;
   
   Z0.pos_tol = Z0.ohm * 0.05;
   Z0.neg_tol = Z0.ohm * 0.05;
#ifdef DEBUG
   if (imp_out_p->model == IMP_L_DIFF_MSTRIP_EMBEDDED) {
      printf("Embedded Differential Microstrip\n"
	     "\t(H1=%lg w=%lg t=%lg Er=%lg K=%lg) = %lg\n",
	     H1, w, t, Er, K, Z0.ohm);
   } else {
      printf("Surface Differential Microstrip\n"
	     "\t(H1=%lg w=%lg t=%lg Er=%lg K=%lg) = %lg\n",
	     H1, w, t, Er, K, Z0.ohm);
   }
#endif
   imp_out_p->ohm = Z0;
   return (IMP_L_OK);
}

/* Courtesy of Bill Reeves, AMP -- modified */
static int imp_lib_diff_sline(imp_in_struc *imp_in, imp_out_struc *imp_out_p)
{
   imp_ohm_struc Z0;
   double        H1;
   double        H2;
   double        t; /* line thickness       */
   double        w; /* line width           */
   double        Er;
   double        c;
   double        K;
   double        w1;
   double        T1, T2;
   
   IMP_ASSERT(imp_in != NULL && imp_out_p != NULL);

   H1 = IMP_M_DIFF_SLINE_H1(imp_in).thick;
   H2 = IMP_M_DIFF_SLINE_H2(imp_in).thick;
   Er = IMP_M_DIFF_SLINE_Er(imp_in);
   t  = IMP_M_DIFF_SLINE_T(imp_in).thick;
   w  = IMP_M_DIFF_SLINE_W(imp_in).thick;
   c  = IMP_M_SPACING(imp_in);
   
   
   memset(imp_out_p, 0, sizeof(*imp_out_p));
   if (c < 0.01) {
      return (IMP_L_OK);
   }

   w1 = w * (w / c);
   
   T1 = (60 / sqrt(Er)) * (log(8.0 * H1 / (0.67 * M_PI * (0.8 * w1 + t))));
   T2 = (60 / sqrt(Er)) * (log(8.0 * H2 / (0.67 * M_PI * (0.8 * w1 + t))));
   Z0.ohm = (2.0 * T1 * T2) / (T1 + T2) * 1.64;
   Z0.pos_tol = Z0.ohm * 0.05;
   Z0.neg_tol = Z0.ohm * 0.05;
   imp_out_p->ohm = Z0;
   imp_out_p->model = IMP_L_DIFF_SLINE;
#ifdef DEBUG
   printf("Differential stripline\n"
	  "\t(H1=%lg H2= %lg w=%lg t=%lg Er=%lg) = %lg\n",
	  H1, H2, w, t, Er, Z0.ohm);
#endif
   return (IMP_L_OK);
}
